@@ -8,6 +8,7 @@
 #include "treeview.h"
 #include "neovimconnector.h"
 #include "errorwidget.h"
+#include "scrollbar.h"
 #include "shell.h"
 
 namespace NeovimQt {
@@ -57,6 +58,7 @@ private slots:
 	void extTablineSet(bool);
 	void changeTab(int index);
 	void saveWindowGeometry();
+
 private:
 	void init(NeovimConnector *);
 	NeovimConnector *m_nvim;
@@ -75,6 +77,7 @@ private:
 	QAction *m_actCopy;
 	QAction *m_actPaste;
 	QAction *m_actSelectAll;
+	ScrollBar *m_scrollbar;
 };
 
 } // Namespace
