@@ -1,5 +1,7 @@
 #include "shell.h"
 
+#include <cmath>
+#include <QFontDialog>
 #include <QPainter>
 #include <QPaintEvent>
 #include <QDebug>
@@ -17,16 +19,9 @@
 namespace NeovimQt {
 
 Shell::Shell(NeovimConnector *nvim, ShellOptions opts, QWidget *parent)
-:ShellWidget(parent), m_attached(false), m_nvim(nvim),
-	m_font_bold(false), m_font_italic(false), m_font_underline(false), m_font_undercurl(false),
-	m_mouseHide(true),
-	m_hg_foreground(Qt::black), m_hg_background(Qt::white), m_hg_special(QColor()),
-	m_cursor_color(Qt::white), m_cursor_pos(0,0), m_insertMode(false),
-	m_resizing(false),
-	m_mouse_wheel_delta_fraction(0, 0),
-	m_neovimBusy(false),
-	m_options(opts),
-	m_mouseEnabled(true)
+	: ShellWidget{ parent }
+	, m_nvim{ nvim }
+	, m_options{ opts }
 {
 	setAttribute(Qt::WA_KeyCompression, false);
 
@@ -49,7 +44,7 @@ Shell::Shell(NeovimConnector *nvim, ShellOptions opts, QWidget *parent)
 	m_pum.setParent(this);
 	m_pum.hide();
 
-	if (m_nvim == NULL) {
+	if (!m_nvim) {
 		qWarning() << "Received NULL as Neovim Connector";
 		return;
 	}
@@ -82,10 +77,14 @@ void Shell::fontError(const QString& msg)
 QString Shell::fontDesc()
 {
 	QString fdesc = QString("%1:h%2").arg(fontFamily()).arg(fontSize());
-	if (font().bold()) {
+	if (font().weight() == QFont::Bold) {
 		fdesc += ":b";
 	} else if (font().weight() == QFont::Light) {
 		fdesc += ":l";
+	} else if (font().weight() == QFont::DemiBold) {
+		fdesc += ":sb";
+	} else if (font().weight() != QFont::Normal) {
+		fdesc += ":w" + QString::number(font().weight());
 	}
 	if (font().italic()) {
 		fdesc += ":i";
@@ -93,44 +92,85 @@ QString Shell::fontDesc()
 	return fdesc;
 }
 
-/**
- * Set the GUI font, or display the current font
- */
-bool Shell::setGuiFont(const QString& fdesc, bool force)
+/// Set the GUI font, or display the current font
+///
+/// @param updateOption controls update of the guifont option, which should not
+/// be updated when the user calls from 'set guifont=...'.
+bool Shell::setGuiFont(const QString& fdesc, bool force, bool updateOption)
 {
-	QStringList attrs = fdesc.split(':');
-	if (attrs.size() < 1) {
-		m_nvim->api0()->vim_report_error("Invalid font");
+	// Exit early if the font description does not change, prevent loops.
+	if (fdesc.compare(fontDesc(), Qt::CaseInsensitive) == 0) {
 		return false;
 	}
 
-	qreal pointSize = font().pointSizeF();
-	int weight = -1;
-	bool italic = false;
-	foreach(QString attr, attrs) {
-		if (attr.size() >= 2 && attr[0] == 'h') {
-			bool ok = false;
-			qreal height = attr.mid(1).toFloat(&ok);
-			if (!ok) {
-				m_nvim->api0()->vim_report_error("Invalid font height");
-				return false;
-			}
-			pointSize = height;
-		} else if (attr == "b") {
-			weight = QFont::Bold;
-		} else if (attr == "l") {
-			weight = QFont::Light;
-		} else if (attr == "i") {
-			italic = true;
+	bool setShellFontSuccess{ false };
+	const bool isGuiDialogRequest{ fdesc == "*" };
+
+	if (isGuiDialogRequest) {
+		bool fontDialogSuccess = false;
+		QFont font{ QFontDialog::getFont(&fontDialogSuccess, QWidget::font(), this, {},
+			QFontDialog::MonospacedFonts) };
+
+		// The font selection dialog was canceled by the user.
+		if (!fontDialogSuccess) {
+			return false;
 		}
+
+		setShellFontSuccess = setShellFont(font.family(), font.pointSize(), font.weight(),
+			font.italic(), force);
 	}
-	bool ok = setShellFont(attrs.at(0), pointSize, weight, italic, force);
-	if (ok && m_attached) {
-		resizeNeovim(size());
-		m_nvim->api0()->vim_set_var("GuiFont", fontDesc());
+	else {
+		QStringList attrs = fdesc.split(':');
+		if (attrs.size() < 1) {
+			m_nvim->api0()->vim_report_error("Invalid font");
+			return false;
+		}
+
+		qreal pointSize = font().pointSizeF();
+		int weight = -1;
+		bool italic = false;
+		for (const auto& attr : attrs) {
+			if (attr.size() >= 2 && attr[0] == 'h') {
+				bool ok{ false };
+				qreal height = attr.mid(1).toFloat(&ok);
+				if (!ok) {
+					m_nvim->api0()->vim_report_error("Invalid font height");
+					return false;
+				}
+				pointSize = height;
+			} else if (attr == "b") {
+				weight = QFont::Bold;
+			} else if (attr == "l") {
+				weight = QFont::Light;
+			} else if (attr == "sb") {
+				weight = QFont::DemiBold;
+			} else if (attr.length() > 0 && attr.at(0) == 'w') {
+				weight = (attr.right(attr.length()-1)).toInt();
+			} else if (attr == "i") {
+				italic = true;
+			}
+		}
+
+		setShellFontSuccess = setShellFont(attrs.at(0), pointSize, weight, italic, force);
 	}
 
-	return ok;
+	// Only update the ShellWidget when font changes.
+	if (!setShellFontSuccess || !m_attached) {
+		return false;
+	}
+
+	// Font changed: trigger resize to update the ShellWidget, and update font variables.
+	resizeNeovim(size());
+
+	m_nvim->api0()->vim_set_var("GuiFont", fontDesc());
+
+	// Updating guifont when the user has already called 'set guifont=...' may cause
+	// unwanted recursion. Only update this option for ':GuiFont', and dialog calls.
+	if (isGuiDialogRequest || updateOption) {
+		m_nvim->api0()->vim_set_option("guifont", fontDesc());
+	}
+
+	return true;
 }
 
 Shell::~Shell()
@@ -145,7 +185,9 @@ void Shell::setAttached(bool attached)
 	m_attached = attached;
 	if (attached) {
 		updateWindowId();
+
 		m_nvim->api0()->vim_set_var("GuiFont", fontDesc());
+
 		if (isWindow()) {
 			updateGuiWindowState(windowState());
 		}
@@ -173,32 +215,6 @@ void Shell::setAttached(bool attached)
 	update();
 }
 
-/// The top left corner position (pixel) for the cursor
-QPoint Shell::neovimCursorTopLeft() const
-{
-	return QPoint(m_cursor_pos.x()*cellSize().width(),
-			m_cursor_pos.y()*cellSize().height());
-}
-
-/// Get the area filled by the cursor
-QRect Shell::neovimCursorRect() const
-{
-	return neovimCursorRect(m_cursor_pos);
-}
-
-/// Get the area filled by the cursor at an arbitrary
-/// position
-QRect Shell::neovimCursorRect(QPoint at) const
-{
-	const Cell& c = contents().constValue(at.y(), at.x());
-	bool wide = c.doubleWidth;
-	QRect r(neovimCursorTopLeft(), cellSize());
-	if (wide) {
-		r.setWidth(r.width()*2);
-	}
-	return r;
-}
-
 void Shell::init()
 {
 	// Make sure the connector provides us with an api object
@@ -321,7 +337,7 @@ void Shell::handlePut(const QVariantList& args)
 
 	QString text = m_nvim->decode(args.at(0).toByteArray());
 	if (text.isEmpty() && m_cursor_pos.x() > 0 &&
-	    contents().constValue(m_cursor_pos.y(), m_cursor_pos.x() - 1).doubleWidth) {
+	    contents().constValue(m_cursor_pos.y(), m_cursor_pos.x() - 1).IsDoubleWidth()) {
 		// nvim will seek to the second cell of a wide char and put "",
 		// expecting the cursor position and cell style to be updated properly.
 		// Handle this case.
@@ -349,11 +365,13 @@ void Shell::handlePut(const QVariantList& args)
  */
 void Shell::handleScroll(const QVariantList& args)
 {
-	if (!args.at(0).canConvert<qint64>()) {
+	if (args.size() < 1
+		|| !args.at(0).canConvert<int64_t>()) {
 		qWarning() << "Unexpected arguments for redraw:scroll" << args;
 		return;
 	}
-	qint64 count = args.at(0).toULongLong();
+
+	const int64_t count{ args.at(0).toLongLong() };
 
 	if (m_scroll_region.contains(m_cursor_pos)) {
 		// Schedule cursor region to be repainted
@@ -456,12 +474,7 @@ void Shell::handleRedraw(const QByteArray& name, const QVariantList& opargs)
 	} else if (name == "mouse_off"){
 		handleMouse(false);
 	} else if (name == "mode_change"){
-		if (opargs.size() < 1 || !opargs.at(0).canConvert<QByteArray>()) {
-			qWarning() << "Unexpected argument for change_mode:" << opargs;
-			return;
-		}
-		QString mode = m_nvim->decode(opargs.at(0).toByteArray());
-		handleModeChange(mode);
+		handleModeChange(opargs);
 	} else if (name == "cursor_on"){
 	} else if (name == "set_title"){
 		handleSetTitle(opargs);
@@ -508,7 +521,7 @@ void Shell::handleRedraw(const QByteArray& name, const QVariantList& opargs)
 	} else if (name == "popupmenu_hide") {
 		m_pum.hide();
 	} else if (name == "mode_info_set") {
-		// TODO
+		handleModeInfoSet(opargs);
 	} else if (name == "flush") {
 		// Do Nothing, a notification that nvim is done redrawing.
 	} else if (name == "grid_resize") {
@@ -527,8 +540,9 @@ void Shell::handleRedraw(const QByteArray& name, const QVariantList& opargs)
 		handleGridCursorGoto(opargs);
 	} else if (name == "grid_scroll") {
 		handleGridScroll(opargs);
-	}
-	else {
+	} else if (name == "hl_group_set") {
+	} else if (name == "win_viewport") {
+	} else {
 		qDebug() << "Received unknown redraw notification" << name << opargs;
 	}
 
@@ -599,15 +613,77 @@ void Shell::handlePopupMenuSelect(const QVariantList& opargs)
 	m_pum.setSelectedIndex(opargs.at(0).toLongLong());
 }
 
-void Shell::setNeovimCursor(quint64 row, quint64 col)
+void Shell::handleModeChange(const QVariantList& opargs)
 {
-	update(neovimCursorRect());
-	m_cursor_pos = QPoint(col, row);
-	update(neovimCursorRect());
-}
+	if (opargs.size() < 2
+		|| !opargs.at(0).canConvert<QByteArray>()
+		|| !opargs.at(1).canConvert<uint64_t>()) {
+		qWarning() << "Unexpected arguments for mode_change:" << opargs;
+		return;
+	}
+
+	const QString mode{ m_nvim->decode(opargs.at(0).toByteArray()) };
+	const uint64_t modeIndex{ opargs.at(1).toULongLong() };
+
+	const uint32_t sizeModeInfo{ static_cast<uint32_t>(m_modeInfo.size()) };
+	if (modeIndex >= sizeModeInfo) {
+		return;
+	}
+
+	const QVariantMap modePropertyMap{ m_modeInfo.at(modeIndex).toMap() };
+
+	Cursor::Shape cursorShape{ Cursor::Shape::Block };
+	const QVariant cursor_shape{ modePropertyMap.value("cursor_shape") };
+	if (!cursor_shape.isNull() && cursor_shape.canConvert<QByteArray>()) {
+		const QString cursor_shape{ modePropertyMap.value("cursor_shape").toByteArray() };
+		if (cursor_shape == "block") {
+			cursorShape = Cursor::Shape::Block;
+		}
+		else if (cursor_shape == "horizontal") {
+			cursorShape = Cursor::Shape::Horizontal;
+		}
+		else if (cursor_shape == "vertical") {
+			cursorShape = Cursor::Shape::Vertical;
+		}
+	}
+
+	uint64_t cellPercentage{ 100 };
+	const QVariant cell_percentage{ modePropertyMap.value("cell_percentage") };
+	if (!cell_percentage.isNull() && cell_percentage.canConvert<uint32_t>()) {
+		const uint32_t cell_percentage{ modePropertyMap.value("cell_percentage").toUInt() };
+		if (cell_percentage > 0 && cell_percentage < 100) {
+			cellPercentage = cell_percentage;
+		}
+	}
+
+	uint64_t blinkWaitTime{ 0 };
+	const QVariant blinkwait{ modePropertyMap.value("blinkwait") };
+	if (!blinkwait.isNull() && blinkwait.canConvert<uint32_t>()) {
+		blinkWaitTime = blinkwait.toUInt();
+	}
+
+	uint64_t blinkOffTime{ 0 };
+	const QVariant blinkoff{ modePropertyMap.value("blinkoff") };
+	if (!blinkoff.isNull() && blinkoff.canConvert<uint32_t>()) {
+		blinkOffTime = blinkoff.toUInt();
+	}
+
+	uint64_t blinkOnTime{ 0 };
+	const QVariant blinkon{ modePropertyMap.value("blinkon") };
+	if (!blinkon.isNull() && blinkon.canConvert<uint32_t>()) {
+		blinkOnTime = blinkon.toUInt();
+	}
+
+	const QVariant attr_id{ modePropertyMap.value("attr_id") };
+	HighlightAttribute highlight;
+	if (!attr_id.isNull() && attr_id.canConvert<uint32_t>()) {
+		highlight = m_highlightMap.value(attr_id.toUInt());
+	}
+
+	m_cursor.SetColor(highlight);
+	m_cursor.SetStyle(cursorShape, cellPercentage);
+	m_cursor.SetTimer(blinkWaitTime, blinkOnTime, blinkOffTime);
 
-void Shell::handleModeChange(const QString& mode)
-{
 	auto old = m_insertMode;
 
 	// TODO: Implement visual aids for other modes
@@ -623,6 +699,30 @@ void Shell::handleModeChange(const QString& mode)
 	}
 }
 
+void Shell::handleModeInfoSet(const QVariantList& opargs)
+{
+	if (opargs.size() < 2
+		|| !opargs.at(0).canConvert<bool>()
+		|| static_cast<QMetaType::Type>(opargs.at(1).type()) != QMetaType::QVariantList) {
+		qWarning() << "Unexpected arguments for mode_info_set:" << opargs;
+		return;
+	}
+
+	const bool cursor_style_enabled{ opargs.at(0).toBool() };
+	const QVariantList mode_info{ opargs.at(1) };
+
+	// Neovim sends mode_info with an extra list container. We should extract this list so that
+	// lower levels do not need to handle the extra type checks.
+	if (mode_info.size() < 1
+		|| static_cast<QMetaType::Type>(mode_info.at(0).type()) != QMetaType::QVariantList) {
+		qWarning() << "Unexpected format for mode_info";
+		return;
+	}
+
+	m_cursor.SetIsEnabled(cursor_style_enabled);
+	m_modeInfo = mode_info.at(0).toList();
+}
+
 void Shell::handleSetTitle(const QVariantList& opargs)
 {
 	if (opargs.size() < 1 || !opargs.at(0).canConvert<QByteArray>()) {
@@ -635,15 +735,15 @@ void Shell::handleSetTitle(const QVariantList& opargs)
 
 void Shell::handleBusy(bool busy)
 {
+	m_cursor.SetIsBusy(busy);
+
 	if (busy != m_neovimBusy) {
 		update(neovimCursorRect());
 	}
+
 	m_neovimBusy = busy;
-	if (busy) {
-		this->setCursor(Qt::WaitCursor);
-	} else {
-		this->unsetCursor();
-	}
+
+	setCursorFromBusyState();
 	emit neovimBusy(busy);
 }
 
@@ -653,13 +753,7 @@ void Shell::handleNeovimNotification(const QByteArray &name, const QVariantList&
 	if (name == "Gui" && args.size() > 0) {
 		QString guiEvName = m_nvim->decode(args.at(0).toByteArray());
 		if (guiEvName == "Font") {
-			if (args.size() == 2) {
-				QString fdesc = m_nvim->decode(args.at(1).toByteArray());
-				setGuiFont(fdesc);
-			} else if (args.size() == 3) {
-				QString fdesc = m_nvim->decode(args.at(1).toByteArray());
-				setGuiFont(fdesc, args.at(2) == 1);
-			}
+			handleGuiFontFunction(args);
 		} else if (guiEvName == "Foreground" && args.size() == 1) {
 			activateWindow();
 			raise();
@@ -737,31 +831,7 @@ void Shell::handleNeovimNotification(const QByteArray &name, const QVariantList&
 		return;
 	}
 
-	foreach(const QVariant& update_item, args) {
-		if ((QMetaType::Type)update_item.type() != QMetaType::QVariantList) {
-			qWarning() << "Received unexpected redraw operation" << update_item;
-			continue;
-		}
-
-		const QVariantList& redrawupdate = update_item.toList();
-		if (redrawupdate.size() < 2) {
-			qWarning() << "Received unexpected redraw operation" << update_item;
-			continue;
-		}
-
-		const QByteArray& name = redrawupdate.at(0).toByteArray();
-		const QVariantList& update_args = redrawupdate.mid(1);
-
-		foreach (const QVariant& opargs_var, update_args) {
-			if ((QMetaType::Type)opargs_var.type() != QMetaType::QVariantList) {
-				qWarning() << "Received unexpected redraw arguments, expecting list" << opargs_var;
-				continue;
-			}
-
-			const QVariantList& opargs = opargs_var.toList();
-			handleRedraw(name, opargs);
-		}
-	}
+	DispatchRedrawNotifications(this, args);
 }
 
 void Shell::handleExtGuiOption(const QString& name, const QVariant& value)
@@ -781,7 +851,7 @@ void Shell::handleExtGuiOption(const QString& name, const QVariant& value)
 void Shell::handleSetOption(const QString& name, const QVariant& value)
 {
 	if (name == "guifont") {
-		setGuiFont(value.toString());
+		setGuiFont(value.toString(), false /*force*/, false /*setOption*/);
 	} else if (name == "guifontset") {
 	} else if (name == "guifontwide") {
 	} else if (name == "linespace") {
@@ -813,6 +883,25 @@ void Shell::handleMouse(bool enabled)
 	m_mouseEnabled = enabled;
 }
 
+void Shell::handleGuiFontFunction(const QVariantList& args)
+{
+	if (args.size() < 2
+		|| !args.at(1).canConvert<QByteArray>())
+	{
+		return;
+	}
+
+	const QString fdesc{ m_nvim->decode(args.at(1).toByteArray()) };
+
+	bool force{ false };
+	if (args.size() >= 3 && args.at(2).canConvert<bool>())
+	{
+		force = args.at(2).toBool();
+	}
+
+	setGuiFont(fdesc, force, true /*setOption*/);
+}
+
 void Shell::handleGridResize(const QVariantList& opargs)
 {
 	if (opargs.size() < 3
@@ -884,30 +973,9 @@ void Shell::handleHighlightAttributeDefine(const QVariantList& opargs)
 	//     "cterm_attr" = opargs.at(2).toMap()
 
 	const uint64_t id = opargs.at(0).toULongLong();
-	const auto& rgb_attr = opargs.at(1).toMap();
-
-	const QColor foregroundColor = (rgb_attr.contains("foreground")) ?
-		color(rgb_attr.value("foreground").toLongLong(), QColor::Invalid) :
-		QColor::Invalid;
-
-	const QColor backgroundColor = (rgb_attr.contains("background")) ?
-		color(rgb_attr.value("background").toLongLong(), QColor::Invalid) :
-		QColor::Invalid;
-
-	const QColor specialColor = rgb_attr.contains(("special")) ?
-		color(rgb_attr.value("special").toLongLong(), QColor::Invalid) :
-		QColor::Invalid;
-
-	HighlightAttribute hl_attr {
-		foregroundColor,
-		backgroundColor,
-		specialColor,
-		rgb_attr.contains("reverse"),
-		rgb_attr.contains("italic"),
-		rgb_attr.contains("bold"),
-		rgb_attr.contains("underline"),
-		rgb_attr.contains("undercurl") };
+	const QVariantMap rgb_attr = opargs.at(1).toMap();
 
+	HighlightAttribute hl_attr{ rgb_attr };
 	m_highlightMap.insert(id, hl_attr);
 }
 
@@ -957,15 +1025,7 @@ void Shell::handleGridLine(const QVariantList& opargs)
 		// Send GUI updates to 'ShellWidget'.
 		for (uint64_t i=0;i<repeat;i++)
 		{
-			put(text, row, col_next,
-				hl_attr.GetForegroundColor(),
-				hl_attr.GetBackgroundColor(),
-				hl_attr.GetSpecialColor(),
-				hl_attr.IsBold(),
-				hl_attr.IsItalic(),
-				hl_attr.IsUnderline(),
-				hl_attr.IsUndercurl());
-
+			put(text, row, col_next, hl_attr);
 			col_next++;
 		}
 	}
@@ -993,13 +1053,13 @@ void Shell::handleGridCursorGoto(const QVariantList& opargs)
 void Shell::handleGridScroll(const QVariantList& opargs)
 {
 	if (opargs.size() < 7
-		|| !opargs.at(0).canConvert<quint64>()
-		|| !opargs.at(1).canConvert<quint64>()
-		|| !opargs.at(2).canConvert<quint64>()
-		|| !opargs.at(3).canConvert<quint64>()
-		|| !opargs.at(4).canConvert<quint64>()
-		|| !opargs.at(5).canConvert<quint64>()
-		|| !opargs.at(6).canConvert<quint64>()) {
+		|| !opargs.at(0).canConvert<uint64_t>()
+		|| !opargs.at(1).canConvert<uint64_t>()
+		|| !opargs.at(2).canConvert<uint64_t>()
+		|| !opargs.at(3).canConvert<uint64_t>()
+		|| !opargs.at(4).canConvert<uint64_t>()
+		|| !opargs.at(5).canConvert<uint64_t>()
+		|| !opargs.at(6).canConvert<int64_t>()) {
 		qWarning() << "Unexpected arguments for grid_scroll:" << opargs;
 		return;
 	}
@@ -1008,21 +1068,24 @@ void Shell::handleGridScroll(const QVariantList& opargs)
 	//     "grid" = opargs.at(0).toULongLong()
 	//     "cols" = opargs.at(6).toULongLong()
 
-	const uint64_t top = opargs.at(1).toULongLong();
-	const uint64_t bot = opargs.at(2).toULongLong();
-	const uint64_t left = opargs.at(3).toULongLong();
-	const uint64_t right = opargs.at(4).toULongLong();
-	const uint64_t rows = opargs.at(5).toULongLong();
+	const uint64_t top{ opargs.at(1).toULongLong() };
+	const uint64_t bot{ opargs.at(2).toULongLong() };
+	const uint64_t left{ opargs.at(3).toULongLong() };
+	const uint64_t right{ opargs.at(4).toULongLong() };
+	const int64_t rows{ opargs.at(5).toLongLong() };
 
 	m_scroll_region = QRect(QPoint(left, top), QPoint(right, bot));
 
+	// Remove old cursor
 	if (m_scroll_region.contains(m_cursor_pos)) {
-		// Schedule cursor region to be repainted
 		update(neovimCursorRect());
 	}
 
 	scrollShellRegion(m_scroll_region.top(), m_scroll_region.bottom(),
 		m_scroll_region.left(), m_scroll_region.right(), rows);
+
+	// Draw new cursor
+	update(neovimCursorRect());
 }
 
 void Shell::paintEvent(QPaintEvent *ev)
@@ -1033,13 +1096,20 @@ void Shell::paintEvent(QPaintEvent *ev)
 		return;
 	}
 
+	// Option guicursor can be disabled with `:set guicursor=`.
+	if (m_cursor.IsEnabled())
+	{
+		ShellWidget::paintEvent(ev);
+		return;
+	}
+
 	ShellWidget::paintEvent(ev);
 
 	// paint cursor - we are not actually using Neovim colors yet,
 	// just invert the shell colors by painting white with XoR
 	if (!m_neovimBusy && ev->region().contains(neovimCursorTopLeft())) {
 		bool wide = contents().constValue(m_cursor_pos.y(),
-						m_cursor_pos.x()).doubleWidth;
+						m_cursor_pos.x()).IsDoubleWidth();
 		QRect cursorRect(neovimCursorTopLeft(), cellSize());
 
 		if (m_insertMode) {
@@ -1070,7 +1140,12 @@ void Shell::keyPressEvent(QKeyEvent *ev)
 		this->setCursor(Qt::BlankCursor);
 	}
 
-	QString inp = Input.convertKey(ev->text(), ev->key(), ev->modifiers());
+	const QString inp{ Input::convertKey(*ev) };
+
+	// Uncomment for key input debugging and unit test writing.
+	// qDebug() << "QKeyEvent ev:" << ev;
+	// qDebug() << "  " << inp;
+
 	if (inp.isEmpty()) {
 		QWidget::keyPressEvent(ev);
 		return;
@@ -1100,9 +1175,9 @@ void Shell::neovimMouseEvent(QMouseEvent *ev)
 		} else {
 			return;
 		}
-		inp = Input.convertMouse(bt, ev->type(), ev->modifiers(), pos, 0);
+		inp = Input::convertMouse(bt, ev->type(), ev->modifiers(), pos, 0);
 	} else {
-		inp = Input.convertMouse(ev->button(), ev->type(), ev->modifiers(), pos,
+		inp = Input::convertMouse(ev->button(), ev->type(), ev->modifiers(), pos,
 						m_mouseclick_count);
 	}
 	if (inp.isEmpty()) {
@@ -1146,9 +1221,11 @@ void Shell::mouseReleaseEvent(QMouseEvent *ev)
 {
 	neovimMouseEvent(ev);
 }
+
 void Shell::mouseMoveEvent(QMouseEvent *ev)
 {
-	unsetCursor();
+	setCursorFromBusyState();
+
 	QPoint pos(ev->x()/cellSize().width(),
 			ev->y()/cellSize().height());
 	if (pos != m_mouse_pos) {
@@ -1158,52 +1235,84 @@ void Shell::mouseMoveEvent(QMouseEvent *ev)
 	}
 }
 
+void Shell::setCursorFromBusyState() noexcept
+{
+	unsetCursor();
+
+	if (m_neovimBusy) {
+		setCursor(Qt::CursorShape::WaitCursor);
+	}
+}
+
+static int normalize(float x)
+{
+	if (x == 0.0f) {
+		return 0;
+	}
+	if (x < 0) {
+		return -1;
+	}
+	return 1;
+}
+
 void Shell::wheelEvent(QWheelEvent *ev)
 {
 	if (!m_attached || !m_mouseEnabled) {
 		return;
 	}
-#ifdef Q_OS_MAC
-	// For some reason <ScrollWheel*> scrolls multiple lines at once
-	// we have to account for it, to make sure that pixelDelta() is used correctly.
-	const int scroll_columns = 6;
-	const int scroll_rows = 3;
-	// Minimal scroll step
-	int scroll_step_x = cellSize().width() * scroll_columns;
-	int scroll_step_y = cellSize().height() * scroll_rows;
-	// Total scroll delta considering previous events
-	QPoint total_delta = m_mouse_wheel_delta_fraction + ev->pixelDelta();
-	// Delta rounded to a minimal scroll step
-	QPoint cell_delta(total_delta.x() / scroll_step_x, total_delta.y() / scroll_step_y);
-	// Save remainder for future events
-	m_mouse_wheel_delta_fraction = total_delta - QPoint(cell_delta.x() * scroll_step_x, cell_delta.y() * scroll_step_y);
-
-	int horiz = cell_delta.x();
-	int vert = cell_delta.y();
+#if (QT_VERSION < QT_VERSION_CHECK(5, 5, 0))
+	const int defaultDeltasPerStep = 120;
 #else
-	int horiz, vert;
-	horiz = ev->angleDelta().x();
-	vert = ev->angleDelta().y();
+	const int defaultDeltasPerStep = QWheelEvent::DefaultDeltasPerStep;
 #endif
 
-	if (horiz == 0 && vert == 0) {
+	QPointF scroll_delta_f(ev->angleDelta());
+	scroll_delta_f /= defaultDeltasPerStep;
+
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 7, 0))
+	if (ev->inverted()) {
+		scroll_delta_f = -scroll_delta_f;
+	}
+#endif
+
+	// Reset scroll remainder if we change scroll direction;
+	const int direction_x = normalize(scroll_delta_f.x());
+	if (direction_x != m_scroll_last_direction.x()) {
+		m_scroll_remainder.setX(0.0);
+		m_scroll_last_direction.setX(direction_x);
+	}
+	const int direction_y = normalize(scroll_delta_f.y());
+	if (direction_y != m_scroll_last_direction.y()) {
+		m_scroll_remainder.setY(0.0);
+		m_scroll_last_direction.setY(direction_y);
+	}
+
+	// Scroll distance considering previous events
+	const QPointF total_delta = scroll_delta_f + m_scroll_remainder;
+	const QPointF scroll_delta(std::floor(total_delta.x()),
+			std::floor(total_delta.y()));
+
+	// Store remainder so we can accumulate fine scroll events
+	m_scroll_remainder = total_delta - scroll_delta;
+
+	if (scroll_delta.isNull()) {
 		return;
 	}
 
-	QPoint pos(ev->x()/cellSize().width(),
+	const QPoint pos(ev->x()/cellSize().width(),
 			ev->y()/cellSize().height());
 
 	QString inp;
-	if (vert != 0) {
+	if (scroll_delta.y() != 0) {
 		inp += QString("<%1ScrollWheel%2><%3,%4>")
-			.arg(Input.modPrefix(ev->modifiers()))
-			.arg(vert > 0 ? "Up" : "Down")
+			.arg(Input::GetModifierPrefix(ev->modifiers()))
+			.arg(scroll_delta.y() > 0 ? "Up" : "Down")
 			.arg(pos.x()).arg(pos.y());
 	}
-	if (horiz != 0) {
+	if (scroll_delta.x() != 0) {
 		inp += QString("<%1ScrollWheel%2><%3,%4>")
-			.arg(Input.modPrefix(ev->modifiers()))
-			.arg(horiz > 0 ? "Left" : "Right")
+			.arg(Input::GetModifierPrefix(ev->modifiers()))
+			.arg(scroll_delta.x() > 0 ? "Left" : "Right")
 			.arg(pos.x()).arg(pos.y());
 	}
 	m_nvim->api0()->vim_input(inp.toLatin1());
