@@ -9,10 +9,13 @@
 #include <QTimer>
 #include <QUrl>
 #include <QList>
+#include <QMap>
 #include <QMenu>
+
 #include "neovimconnector.h"
+#include "shellwidget/highlight.h"
 #include "shellwidget/shellwidget.h"
-#include "highlight.h"
+#include "shellwidget/cursor.h"
 #include "popupmenu.h"
 #include "popupmenumodel.h"
 
@@ -33,7 +36,7 @@ class ShellOptions {
 public:
 	bool enable_ext_tabline{ true };
 	bool enable_ext_popupmenu{ true };
-	bool enable_ext_linegrid{ false };
+	bool enable_ext_linegrid{ true };
 	int nvim_show_tabline{ 1 };
 };
 
@@ -53,6 +56,11 @@ public:
 	bool neovimAttached() const;
 	QString fontDesc();
 
+	/// Dispatches Neovim redraw notifications to T::handleRedraw
+	template <class T>
+	static void DispatchRedrawNotifications(
+		T* pThis, const QVariantList& args) noexcept;
+
 signals:
 	void neovimTitleChanged(const QString &title);
 	void neovimBusy(bool);
@@ -77,7 +85,7 @@ public slots:
 	void handleNeovimNotification(const QByteArray &name, const QVariantList& args);
 	void resizeNeovim(const QSize&);
 	void resizeNeovim(int n_cols, int n_rows);
-	bool setGuiFont(const QString& fdesc, bool force = false);
+	bool setGuiFont(const QString& fdesc, bool force, bool updateOption);
 	void updateGuiWindowState(Qt::WindowStates state);
 	void openFiles(const QList<QUrl> url);
 
@@ -101,11 +109,6 @@ protected:
 	virtual void wheelEvent(QWheelEvent *event) Q_DECL_OVERRIDE;
 	virtual bool event(QEvent *event) Q_DECL_OVERRIDE;
 
-	QPoint neovimCursorTopLeft() const;
-	QRect neovimCursorRect() const;
-	QRect neovimCursorRect(QPoint at) const;
-	void setNeovimCursor(quint64 col, quint64 row);
-
 	virtual void resizeEvent(QResizeEvent *ev) Q_DECL_OVERRIDE;
 	virtual void keyPressEvent(QKeyEvent *ev) Q_DECL_OVERRIDE;
         void paintLogo(QPainter&);
@@ -122,7 +125,8 @@ protected:
 	virtual void handleHighlightSet(const QVariantMap& args);
 	virtual void handleRedraw(const QByteArray& name, const QVariantList& args);
 	virtual void handleScroll(const QVariantList& args);
-	virtual void handleModeChange(const QString& mode);
+	virtual void handleModeChange(const QVariantList& opargs);
+	virtual void handleModeInfoSet(const QVariantList& opargs);
 	virtual void handleSetTitle(const QVariantList& opargs);
 	virtual void handleSetScrollRegion(const QVariantList& opargs);
 	virtual void handleBusy(bool);
@@ -131,6 +135,7 @@ protected:
 	virtual void handlePopupMenuShow(const QVariantList& opargs);
 	virtual void handlePopupMenuSelect(const QVariantList& opargs);
 	virtual void handleMouse(bool);
+	virtual void handleGuiFontFunction(const QVariantList& args);
 
 	// Modern 'ext_linegrid' Grid UI Events
 	virtual void handleGridResize(const QVariantList& opargs);
@@ -145,6 +150,7 @@ protected:
 	virtual void mouseReleaseEvent(QMouseEvent *ev) Q_DECL_OVERRIDE;
 	virtual void mouseMoveEvent(QMouseEvent *ev) Q_DECL_OVERRIDE;
 	void bailoutIfinputBlocking();
+	void setCursorFromBusyState() noexcept;
 
 	QString neovimErrorToString(const QVariant& err);
 
@@ -152,44 +158,49 @@ private slots:
         void setAttached(bool attached=true);
 
 private:
-	bool m_attached;
-
-	NeovimConnector *m_nvim;
+	bool m_attached{ false };
+	NeovimConnector* m_nvim{ nullptr };
 
 	QList<QUrl> m_deferredOpen;
 
 	QRect m_scroll_region;
-	bool m_font_bold, m_font_italic, m_font_underline, m_font_undercurl;
-	bool m_mouseHide;
+	bool m_font_bold{ false };
+	bool m_font_italic{ false };
+	bool m_font_underline{ false };
+	bool m_font_undercurl{ false };
+	bool m_mouseHide{ true };
 
 	// highlight fg/bg - from redraw:highlightset - by default we
 	// use the values from above
-	QColor m_hg_foreground, m_hg_background, m_hg_special;
-	QColor m_cursor_color;
+	QColor m_hg_foreground{ Qt::black };
+	QColor m_hg_background{ Qt:: white };
+	QColor m_hg_special;
+	QColor m_cursor_color{ Qt::white };
 
 	/// Modern 'ext_linegrid' highlight definition map
 	QMap<uint64_t, HighlightAttribute> m_highlightMap;
 
-	/// Cursor position in shell coordinates
-	QPoint m_cursor_pos;
-	bool m_cursor;
-	bool m_insertMode;
-	bool m_resizing;
+	/// Neovim mode descriptions from "mode_change", used by guicursor
+	QVariantList m_modeInfo;
+
+	bool m_insertMode{ false };
+	bool m_resizing{ false };
 	QSize m_resize_neovim_pending;
-	QLabel *m_tooltip;
+	QLabel* m_tooltip{ nullptr };
 	QPoint m_mouse_pos;
 	// 2/3/4 mouse click tracking
 	QTimer m_mouseclick_timer;
-	short m_mouseclick_count;
+	uint8_t m_mouseclick_count{ 0 };
 	Qt::MouseButton m_mouseclick_pending;
 	// Accumulates remainder of steppy scroll
-	QPoint m_mouse_wheel_delta_fraction;
+	QPointF m_scroll_remainder;
+	QPoint m_scroll_last_direction;
 
 	// Properties
-	bool m_neovimBusy;
+	bool m_neovimBusy{ false };
 	ShellOptions m_options;
 	PopupMenu m_pum{ this };
-	bool m_mouseEnabled;
+	bool m_mouseEnabled{ true };
 };
 
 class ShellRequestHandler: public QObject, public MsgpackRequestHandler
@@ -200,6 +211,34 @@ public:
 	virtual void handleRequest(MsgpackIODevice* dev, quint32 msgid, const QByteArray& method, const QVariantList& args);
 };
 
+template <class T>
+/*static*/ void Shell::DispatchRedrawNotifications(T* pThis, const QVariantList& args) noexcept
+{
+	for(const auto& update_item : args) {
+		if (!update_item.canConvert<QVariantList>()) {
+			qWarning() << "Received unexpected redraw operation" << update_item;
+			continue;
+		}
+
+		const QVariantList& redrawupdate{ update_item.toList() };
+		if (redrawupdate.size() < 2) {
+			qWarning() << "Received unexpected redraw operation" << update_item;
+			continue;
+		}
+
+		const QByteArray& name{ redrawupdate.at(0).toByteArray() };
+		const QVariantList& update_args{ redrawupdate.mid(1) };
+
+		for(const auto& opargs_var : update_args) {
+			if (!opargs_var.canConvert<QVariantList>()) {
+				qWarning() << "Received unexpected redraw arguments, expecting list" << opargs_var;
+				continue;
+			}
+
+			pThis->handleRedraw(name, opargs_var.toList());
+		}
+	}
+}
 
 } // Namespace
 #endif
