@@ -5,6 +5,11 @@
 #include "shellwidget.h"
 #include "helpers.h"
 
+#if defined(Q_OS_WIN) && defined(USE_STATIC_QT)
+#include <QtPlugin>
+Q_IMPORT_PLUGIN (QWindowsIntegrationPlugin);
+#endif
+
 ShellWidget *s = NULL;
 QFile *f = NULL;
 
