@@ -3,7 +3,7 @@
 #include "shellcontents.h"
 #include "konsole_wcwidth.h"
 
-Cell ShellContents::invalidCell = Cell::invalid();;
+/*static*/ Cell ShellContents::invalidCell{ Cell::MakeInvalidCell() };
 
 /// Build shell contents from file, each line in the
 /// file is a shell line.
@@ -65,7 +65,7 @@ void ShellContents::clearAll(QColor bg)
 {
 	for (int i=0; i<_rows; i++) {
 		for (int j=0; j<_columns; j++) {
-			_data[i*_columns+j] = Cell::bg(bg);
+			_data[i*_columns+j] = Cell{ bg };
 		}
 	}
 }
@@ -118,7 +118,7 @@ void ShellContents::clearRegion(int row0, int col0, int row1, int col1,
 
 	for (int i=row0; i<row1; i++) {
 		for (int j=col0; j<col1; j++) {
-			_data[i*_columns + j] = Cell::bg(bg);
+			_data[i*_columns + j] = Cell{ bg };
 		}
 	}
 }
@@ -221,28 +221,29 @@ const Cell& ShellContents::constValue(int row, int column) const
 }
 
 /// Writes content to the shell, returns the number of columns written
-int ShellContents::put(const QString& str, int row, int column,
-		QColor fg, QColor bg, QColor sp, bool bold, bool italic,
-		bool underline, bool undercurl)
+int ShellContents::put(
+	const QString& str,
+	int row,
+	int column,
+	const HighlightAttribute& hl_attr) noexcept
 {
 	if (row < 0 || row >= _rows || column < 0 || column >= _columns) {
 		return 0;
 	}
-	
-	auto vec = str.toUcs4();
+
+	const QVector<uint> vec{ str.toUcs4() };
 
 	int pos = column;
-	foreach(const uint chr, vec) {
-		Cell& c = value(row, pos);
-		c = Cell(chr, fg, bg, sp, bold, italic, underline, undercurl);
-		if (c.doubleWidth) {
-			value(row, pos+1) = Cell();
-			pos += 2;
-		} else {
-			pos += 1;
+	for(const uint chr : vec) {
+		Cell& cell{ value(row, pos) };
+		cell = { chr, hl_attr };
+		pos++;
+
+		// Clear neighboring character for double-width cell.
+		if (cell.IsDoubleWidth()) {
+			value(row, pos) = {};
+			pos++;
 		}
 	}
 	return pos - column;
 }
-
-
