@@ -1,18 +1,17 @@
-#ifndef QSHELLWIDGET2_SHELLCONTENTS
-#define QSHELLWIDGET2_SHELLCONTENTS
+#pragma once
+
+#include "cell.h"
 
 /// A class to hold the contents of the shell / i.e. a grid of characters. This
 /// class is meant to hold state about shell contents, but no more - e.g. cursor
 /// information should be stored somewhere else.
-#include "cell.h"
-
 class ShellContents
 {
 public:
 	ShellContents(int rows, int columns);
 	~ShellContents();
 	ShellContents(const ShellContents& other);
-	
+
 	inline int columns() const {
 		return _columns;
 	}
@@ -25,10 +24,13 @@ public:
 	const Cell* data();
 	Cell& value(int row, int column);
 	const Cell& constValue(int row, int column) const;
-	int put(const QString&, int row, int column,
-			QColor fg=Qt::black, QColor bg=Qt::white, QColor sp=QColor(),
-			bool bold=false, bool italic=false,
-			bool underline=false, bool undercurl=false);
+
+	/// Insert string `str` into cell grid.
+	int put(
+		const QString& str,
+		int row,
+		int column,
+		const HighlightAttribute& hl_attr = {}) noexcept;
 
 	void clearAll(QColor bg=QColor());
 	void clearRow(int r, int startCol=0);
@@ -49,5 +51,3 @@ private:
 
 	ShellContents& operator=(const ShellContents& other);
 };
-
-#endif
