@@ -5,14 +5,19 @@
 #include <QAbstractSocket>
 #include <QProcess>
 #include <QTextCodec>
+#include "msgpackiodevice.h"
 #include "auto/neovimapi0.h"
 #include "auto/neovimapi1.h"
 #include "auto/neovimapi2.h"
 #include "auto/neovimapi3.h"
+#include "auto/neovimapi4.h"
+#include "auto/neovimapi5.h"
+#include "auto/neovimapi6.h"
 
 namespace NeovimQt {
 
 class MsgpackIODevice;
+class MsgpackRequestHandler;
 class NeovimConnectorHelper;
 class NeovimConnector: public QObject
 {
@@ -20,6 +25,9 @@ class NeovimConnector: public QObject
 	friend class NeovimApi1;
 	friend class NeovimApi2;
 	friend class NeovimApi3;
+	friend class NeovimApi4;
+	friend class NeovimApi5;
+	friend class NeovimApi6;
 	friend class NeovimConnectorHelper;
 	Q_OBJECT
 	/**
@@ -76,15 +84,21 @@ public:
 	NeovimApi1 * api1();
 	NeovimApi2 * api2();
 	NeovimApi3 * api3();
+	NeovimApi4 * api4();
+	NeovimApi5 * api5();
+	NeovimApi6 * api6();
 	uint64_t channel();
 	QString decode(const QByteArray&);
 	QByteArray encode(const QString&);
 	NeovimConnectionType connectionType();
 	/** Some requests for metadata and ui attachment enforce a timeout in ms */
 	void setRequestTimeout(int);
+	/** Set a handler for msgpack rpc requests **/
+	void setRequestHandler(MsgpackRequestHandler *);
 
 	quint64 apiCompatibility();
 	quint64 apiLevel();
+	bool hasUIOption(const QByteArray& option);
 
 signals:
 	/** Emitted when Neovim is ready @see ready */
@@ -106,27 +120,31 @@ protected slots:
 	void msgpackError();
 
 private:
-	MsgpackIODevice *m_dev;
-	NeovimConnectorHelper *m_helper;
+	MsgpackIODevice *m_dev{ nullptr };
+	NeovimConnectorHelper *m_helper{ nullptr };
 	QString m_errorString;
-	NeovimError m_error;
+	NeovimError m_error{ NoError };
 
-	NeovimApi0 *m_api0;
-	NeovimApi1 *m_api1;
-	NeovimApi2 *m_api2;
-	NeovimApi3 *m_api3;
-	quint64 m_channel;
-	quint64 m_api_compat;
-	quint64 m_api_supported;
+	NeovimApi0 *m_api0{ nullptr };
+	NeovimApi1 *m_api1{ nullptr };
+	NeovimApi2 *m_api2{ nullptr };
+	NeovimApi3 *m_api3{ nullptr };
+	NeovimApi4 *m_api4{ nullptr };
+	NeovimApi5 *m_api5{ nullptr };
+	NeovimApi6 *m_api6{ nullptr };
+	quint64 m_channel{ 0 };
+	quint64 m_api_compat{ 0 };
+	quint64 m_api_supported{ 0 };
 
 	// Store connection arguments for reconnect()
-	NeovimConnectionType m_ctype;
+	NeovimConnectionType m_ctype{ OtherConnection };
 	QStringList m_spawnArgs;
 	QString m_spawnExe;
 	QString m_connSocket, m_connHost;
+	QVariantList m_uiOptions;
 	int m_connPort;
-	bool m_ready;
-	int m_timeout;
+	bool m_ready{ false };
+	int m_timeout{ 20000 };
 };
 } // namespace NeovimQt
 Q_DECLARE_METATYPE(NeovimQt::NeovimConnector::NeovimError)
