@@ -54,13 +54,13 @@ private slots:
 	}
 
 	void connectToNeovimTCP() {
-		// These 2 cases WILL FAIL because there is no Neovim instance running
 		NeovimConnector *c = NeovimConnector::connectToNeovim("127.0.0.1:64999");
 		QCOMPARE(c->connectionType(), NeovimConnector::HostConnection);
 		QSignalSpy onError(c, SIGNAL(error(NeovimError)));
 		QVERIFY(onError.isValid());
-		// The signal might be emited before we get to connect
-		SPYWAIT(onError);
+
+		// Pull Request #612: On MacOS, this test can fail without a long timeout value.
+		QVERIFY(SPYWAIT(onError, 120000 /*msec*/));
 
 		QCOMPARE(c->errorCause(), NeovimConnector::SocketError);
 		c->deleteLater();
@@ -71,8 +71,9 @@ private slots:
 		QCOMPARE(c->connectionType(), NeovimConnector::SocketConnection);
 		QSignalSpy onError(c, SIGNAL(error(NeovimError)));
 		QVERIFY(onError.isValid());
-		// The signal might be emited before we get to connect
-		SPYWAIT(onError);
+
+		// Test Performance: timeout expected, set value carefully.
+		QVERIFY(!SPYWAIT(onError, 5000 /*msec*/));
 
 		QCOMPARE(c->errorCause(), NeovimConnector::SocketError);
 		c->deleteLater();
@@ -99,9 +100,15 @@ private slots:
 			QString("%1:%2")
 				.arg(server->serverAddress().toString())
 				.arg(server->serverPort()));
+
+		// Test Performance: timeout expected, set value carefully.
+		c->setRequestTimeout(1000 /*msec*/);
+
 		QSignalSpy onError(c, SIGNAL(error(NeovimError)));
 		QVERIFY(onError.isValid());
-		QVERIFY(SPYWAIT2(onError));
+
+		QVERIFY(SPYWAIT(onError, 5000 /*msec*/));
+
 		QCOMPARE(c->errorCause(), NeovimConnector::RuntimeMsgpackError);
 		c->deleteLater();
 	}
